@@ -702,7 +702,7 @@ WHERE
         parameters = None
 
         if template[KG__email_template__type] in [EMAIL_TYPE__signup, EMAIL_TYPE__reset_password, EMAIL_TYPE__unregistered_password_reset]:
-            if not template[KG__email_template__requires_confirmation]:
+            if template[KG__email_template__requires_confirmation]:
                 self.add_account_password(sec_evt[KG__security_event__account], inputs[KEY__password])
 
         # Anything where we follow an email link confirms the account
@@ -923,7 +923,7 @@ WHERE
 
         self._send_signup_email(sign_up_template, account_id, app, username)
 
-    def sign_up(self, inputs):
+    def sign_up(self, inputs, ip_address: str, response: JAAQLResponse):
         app = application__select(self.jaaql_lookup_connection, inputs[KG__security_event__application])
 
         if inputs[KEY__sign_up_template] is None:
@@ -979,6 +979,9 @@ WHERE
 
         self._send_signup_email(sign_up_template, new_account_id, app, inputs[KEY__username])
 
+        if not requires_confirmation:
+            self.get_auth_token(inputs[KEY__username], ip_address, password, response, True)
+
         return res
 
     def invite(self, inputs: dict, account_id: str, is_the_anonymous_user: bool):
@@ -1194,6 +1197,8 @@ WHERE
 
         try:
             res = json.loads(result.stdout)
+            if os.environ.get("JAAQL_PROCEDURE_DEBUGGING", "false").lower() == "true":
+                print(result.stderr)
             if result.returncode == 0:
                 return res
             elif "error_code" in res:
@@ -1209,7 +1214,7 @@ WHERE
                 raise UnhandledRemoteProcedureError()
 
             traceback.print_exc()
-            raise HttpStatusException("Could not intepret webhook procedure result", HTTPStatus.INTERNAL_SERVER_ERROR)
+            raise HttpStatusException("Could not intepret remote procedure result", HTTPStatus.INTERNAL_SERVER_ERROR)
 
     def handle_webhook(self, application: str, name: str, body: bytes, headers: dict, args: dict, response: JAAQLResponse):
         rpc = remote_procedure__select(self.jaaql_lookup_connection, application, name)
